/*
 * Copyright (c) 2009, Red Hat Middleware LLC or third-party contributors as
 * indicated by the @author tags or express copyright attribution
 * statements applied by the authors.  All third-party contributions are
 * distributed under license by Red Hat Middleware LLC.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License
 * for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution; if not, write to:
 * Free Software Foundation, Inc.
 * 51 Franklin Street, Fifth Floor
 * Boston, MA  02110-1301  USA
 */
package org.jboss.maven.util.file;

import java.io.File;
import java.io.IOException;

import org.codehaus.plexus.util.DirectoryScanner;
import org.codehaus.plexus.util.FileUtils;

/**
 * Copies a directory with its structure intact.  Optionally allows defining
 * includes and/or excludes.  Internally uses a {@link org.codehaus.plexus.util.DirectoryScanner}, mainly
 * to ensure we filter out "standard excludes" (like SCM dirs) but also to apply
 * any optional includes and/or excludes.
 *
 * @author Steve Ebersole
 */
public class DirectoryCopier {
	private final File sourceDirectory;

	/**
	 * Instantiates a directory copier over the given source directory.
	 *
	 * @param sourceDirectory The directory from which to copy.
	 */
	public DirectoryCopier(File sourceDirectory) {
		this.sourceDirectory = sourceDirectory;
	}

	/**
	 * Retrieve the source directory.
	 *
	 * @return The source directory.
	 */
	public File getSourceDirectory() {
		return sourceDirectory;
	}

	/**
	 * Copy contents of the {@link #getSourceDirectory() source directory} to the given target directory.
	 * <p/>
	 * Same as {@link #copyTo(File, String[], String[]) copy(targetDirectory,null,null)}
	 *
	 * @param targetDirectory The directory to which to copy the content
	 *
	 * @throws IOException Indicates a problem performing the copy.
	 */
	public void copyTo(File targetDirectory) throws IOException {
		copyTo( targetDirectory, null, null );
	}

	/**
	 * Copy contents of the {@link #getSourceDirectory() source directory} to the given target directory.
	 * <p/>
	 * NOTE: the copier copies all content to the target directory except for<ul>
	 * <li>files not deemed matches as defined by given includes and excludes</li>
	 * <li>SCM-related files (et al.) as defined by {@link DirectoryScanner#addDefaultExcludes()}</li>
	 * </ul>
	 *
	 * @param targetDirectory The directory to which to copy the matching content
	 * @param includes Explicit inclusion match filename patterns.
	 * @param excludes Explicit exclusion match filename patterns.
	 *
	 * @throws IOException Indicates a problem performing the copy.
	 */
	public void copyTo(File targetDirectory, String[] includes, String[] excludes) throws IOException {
		if ( !sourceDirectory.exists() ) {
			return;
		}
		String[] list = sourceDirectory.list();
		if ( list == null || list.length == 0 ) {
			return;
		}

		if ( !targetDirectory.exists() ) {
			boolean created = targetDirectory.mkdirs();
			if ( !created ) {
				throw new IOException( "Filesystem reported it was unable to create directory " + targetDirectory.getAbsolutePath() );
			}
		}

		DirectoryScanner scanner = new DirectoryScanner();
		scanner.setBasedir( sourceDirectory );
		scanner.setIncludes( includes );
		scanner.setExcludes( excludes );
		scanner.addDefaultExcludes();
		scanner.scan();

		for ( String includedFile : scanner.getIncludedFiles() ) {
			copyFileIfModified(
					new File( sourceDirectory, includedFile ),
					new File( targetDirectory, includedFile )
			);
		}
	}

	/**
	 * Copies the source file to the destination file.
	 * 
	 * @param source The file to be copied
	 * @param destination The file to which to copy the source.
	 *
	 * @throws IOException Indicates a problem performing the copy.
	 */
	public static void copyFileIfModified(File source, File destination) throws IOException {
		if ( destination.lastModified() < source.lastModified() ) {
			FileUtils.copyFile( source.getCanonicalFile(), destination );
			// tell IntelliJ to ignore that we ignore the return value from setLastModified()
			//noinspection ResultOfMethodCallIgnored
			destination.setLastModified( source.lastModified() );
		}
	}
}
